from src.core.bt_module import BTModule, BTFile
from src.views.utils import get_view_package_path_from_bt_package
from enum import Enum

from src.utils.config_manager_singleton import ConfigManagerSingleton


class EntityState(str, Enum):
    DELETED = "#Red"
    NEUTRAL = ""
    CREATED = "#Green"


PACKAGE_NAME_SPLITTER = "."


class ViewPackage:
    name = ""
    parent: "ViewPackage" = None
    sub_modules: list["ViewPackage"] = None
    state: EntityState = EntityState.NEUTRAL
    view_dependency_list: list["ViewDependancy"] = None
    bt_package: BTModule = None

    def __init__(self, bt_package: BTModule) -> None:
        self.view_dependency_list = []
        self.name = PACKAGE_NAME_SPLITTER.join(
            get_view_package_path_from_bt_package(bt_package).split("/")
        )
        self.bt_package = bt_package
        self.sub_modules = []

    @property
    def path(self):
        return get_view_package_path_from_bt_package(self.bt_package)

    @property
    def parent_path(self):
        return get_view_package_path_from_bt_package(self.bt_package.parent_module)

    def setup_dependencies(self, view_package_map: dict[str, "ViewPackage"]):
        bt_dependencies = self.bt_package.get_module_dependencies()
        self.parent = view_package_map.get(self.parent_path)
        if self.parent:
            self.parent.sub_modules.append(self)
        for bt_package_dependency in bt_dependencies:
            view_path = get_view_package_path_from_bt_package(bt_package_dependency)
            if view_path == self.path:
                continue
            try:
                view_package_dependency = view_package_map[view_path]
                self.view_dependency_list.append(
                    ViewDependancy(
                        self,
                        view_package_dependency,
                        self.bt_package,
                        bt_package_dependency,
                    )
                )
            except Exception:
                pass

    def get_parent_list(self):
        res = []
        if self.parent is None:
            return res
        res.append(self.parent)
        res.extend(self.parent.get_parent_list())
        return res

    def render_package_pu(self) -> str:
        config_manager = ConfigManagerSingleton()
        state_str = self.state.value
        if self.state == EntityState.NEUTRAL:
            state_str = config_manager.package_color

        return f'package "{self.name}" {state_str}'

    def render_dependency_pu(self) -> str:
        return "\n".join(
            [
                view_dependency.render_pu()
                for view_dependency in self.view_dependency_list
            ]
        )

    def render_package_json(self):
        return {"name": self.name, "state": self.state.name}

    def render_dependency_json(self):
        return [
            view_dependency.render_json()
            for view_dependency in self.view_dependency_list
        ]

    def filter_excess_packages_dependencies(self, used_packages: set["ViewPackage"]):
        for sub_module in self.sub_modules:
            if sub_module not in used_packages:
                sub_module.filter_excess_packages_dependencies(used_packages)
                self.view_dependency_list.extend(sub_module.view_dependency_list)

        for dependency in self.view_dependency_list:
            parent_list = dependency.to_package.get_parent_list()
            for parent in parent_list:
                if parent in used_packages:
                    dep = ViewDependancy(
                        self,
                        parent,
                        dependency.from_bt_package,
                        dependency.to_bt_package,
                    )
                    self.view_dependency_list.append(dep)
                    break

        # change all from package to self
        for dependency in self.view_dependency_list:
            dependency.from_package = self

        # filter all dependencies pointing to self
        self.view_dependency_list = [
            dependency
            for dependency in self.view_dependency_list
            if dependency.to_package != self and dependency.to_package in used_packages
        ]

        # Makes sure that there is only one dependency per package
        aggregate_dependency_map: dict[str, ViewDependancy] = {}
        for dependency in self.view_dependency_list:
            if dependency.id not in aggregate_dependency_map:
                aggregate_dependency_map[dependency.id] = dependency
            else:
                dep = aggregate_dependency_map[dependency.id]
                dep.dependency_count += dependency.dependency_count
        self.view_dependency_list = list(aggregate_dependency_map.values())

    def get_dependency_map(self) -> dict[str, "ViewDependancy"]:
        return {
            dependency.to_package.path: dependency
            for dependency in self.view_dependency_list
        }


class ViewDependancy:
    state: EntityState = EntityState.NEUTRAL

    from_package: ViewPackage = None
    to_package: ViewPackage = None

    from_bt_package: BTModule = None
    to_bt_package: BTModule = None

    edge_files: list[tuple[BTFile, BTFile]] = []

    dependency_count = 0

    render_diff = {}

    def __init__(
        self,
        from_package: ViewPackage,
        to_package: ViewPackage,
        from_bt_package: BTModule,
        to_bt_package: BTModule,
    ) -> None:
        self.from_package = from_package
        self.to_package = to_package
        self.from_bt_package = from_bt_package
        self.to_bt_package = to_bt_package
        self.dependency_count = self.from_bt_package.get_dependency_count(
            self.to_bt_package
        )
        self.edge_files = self.from_bt_package.get_file_level_relations(
            self.to_bt_package
        )

    @property
    def id(self):
        return f"{self.from_package.name}-->{self.to_package.name}"

    def render_pu(self) -> str:
        config_manager = ConfigManagerSingleton()

        if not self.render_diff:
            dependency_count_str = ""
            if config_manager.show_dependency_count:
                dependency_count_str = f": {self.dependency_count}"
            from_name = self.from_package.name
            to_name = self.to_package.name
            return (
                f'"{from_name}"-->"{to_name}" {self.state.value} {dependency_count_str}'
            )
        else:
            return f'"{self.render_diff["from_package"].name}"-->"{self.render_diff["to_package"].name}" {self.render_diff["color"].value} : {self.render_diff["label"]}'

    def render_json(self) -> dict:
        config_manager = ConfigManagerSingleton()

        if not self.render_diff:
            label = ""
            if config_manager.show_dependency_count:
                label = f"{self.dependency_count}"
            from_package: ViewPackage = self.from_package.name
            to_package: ViewPackage = self.to_package.name
            label = str(self.dependency_count)
            state: EntityState = self.state
            state_str = state.name
        else:
            from_package: ViewPackage = self.render_diff["from_package"].name
            to_package: ViewPackage = self.render_diff["to_package"].name
            label = self.render_diff["label"]
            state: EntityState = self.render_diff["color"]
            state_str = state.name
        return {
            "state": state_str,
            "fromPackage": from_package,
            "toPackage": to_package,
            "label": label,
            "relations": [
                {
                    "from_file": {"name": relation[0].label, "path": relation[0].file},
                    "to_file": {"name": relation[1].label, "path": relation[1].file},
                }
                for relation in self.edge_files
            ],
        }



import json

import astroid
import sys
import os

from src.core.bt_file import BTFile, get_imported_modules
from src.core.bt_module import BTModule
from astroid.manager import AstroidManager


class BTGraph:
    DEFAULT_SETTINGS = {"diagram_name": "", "project": None}
    root_module_location: str = None
    target_project_base_location: str = None
    root_module = None
    base_module = None
    am: AstroidManager = None

    def __init__(self, am: AstroidManager) -> None:
        self.am = am

    def build_graph(self, config: dict):
        config_path = config.get("_config_path")
        self.root_module_location = os.path.join(config_path, config.get("rootFolder"))
        self.target_project_base_location = config_path

        sys.path.insert(0, config_path)
        sys.path.insert(1, self.root_module_location)

        bt_module_list: list[BTModule] = []

        # Read all the python files within the project
        file_list = self._get_files_recursive(self.root_module_location)

        # Create modules and add them to module list (No relations between them yet)
        for file in file_list:
            try:
                if not file.endswith("__init__.py"):
                    continue
                bt_module = BTModule(file, self.am)
                bt_module.add_files()
                bt_module_list.append(bt_module)
            except Exception as e:
                print(e)
                continue

        # Add relations between the modules (parent and child nodes)
        for module in bt_module_list:
            for parent_module in bt_module_list:
                if module == parent_module:
                    continue
                if parent_module.path == os.path.dirname(module.path):
                    parent_module.child_module.append(module)
                    module.parent_module = parent_module

        # Find the root node
        self.root_module = next(
            filter(lambda e: e.parent_module is None, bt_module_list)
        )
        self.base_module = self.root_module

        # Add dependencies between all the files
        btf_map = self.get_all_bt_files_map()

        for bt_file in btf_map.values():
            imported_modules = get_imported_modules(
                bt_file.ast, self.target_project_base_location, self.am
            )
            bt_file >> [
                btf_map[module.file]
                for module in imported_modules
                if module.file in btf_map
            ]

        sys.path = sys.path[2:]
        astroid.manager.AstroidManager().clear_cache()
        self.am.clear_cache()

    def get_bt_file(self, path: str) -> BTFile:
        file_path = self.am.ast_from_module_name(path).file
        bt_file = self.get_all_bt_files_map()[file_path]
        return bt_file

    def get_bt_module(self, path: str) -> BTModule:
        path_list = path.split(".")[1:]
        current_module = self.root_module
        while path_list:
            current_module = next(
                filter(
                    lambda e: e.name == path_list[0],
                    current_module.child_module,
                )
            )
            path_list.pop(0)
        return current_module

    def change_scope(self, path: str):
        self.root_module = self.get_bt_module(path)

    def get_all_bt_files_map(self) -> dict[str, BTFile]:
        return {btf.file: btf for btf in self.root_module.get_files_recursive()}

    def get_all_bt_modules_map(self) -> dict[str, BTModule]:
        return {btm.path: btm for btm in self.root_module.get_submodules_recursive()}

    def _get_files_recursive(self, path: str) -> list[str]:
        file_list = []
        t = list(os.walk(path))
        for root, _, files in t:
            for file in files:
                file_list.append(os.path.join(root, file))

        return file_list




import sys
from src.core.bt_graph import BTGraph
from src.views.view_entities import (
    PACKAGE_NAME_SPLITTER,
    EntityState,
    ViewPackage,
)
from src.views.utils import get_view_package_path_from_bt_package
import os
from typing import Callable


def render_views(
    graph: BTGraph,
    config: dict,
    save_to_file: Callable[[list[ViewPackage], str, dict], None],
):
    views = _create_view_graphs(graph, config)
    for view_name, view_package_map in views.items():
        if os.getenv("MT_DEBUG"):
            dep_count = sum(
                len(package.view_dependency_list)
                for package in view_package_map.values()
            )
            package_count = len(list(view_package_map.values()))
            print("View name:", view_name)
            print("Package count:", package_count)
            print("Dependency count:", dep_count)

        view_graph = list(view_package_map.values())
        view_config: dict = config["views"][view_name]
        use_package_path_as_label = view_config.get("usePackagePathAsLabel", True)
        if not use_package_path_as_label:
            _handle_duplicate_name(view_graph)

        save_to_file(view_graph, view_name, config)


def render_diff_views(
    local_bt_graph: BTGraph,
    remote_bt_graph: BTGraph,
    config: dict,
    save_to_file: Callable[[list[ViewPackage], str, dict], None],
):
    local_graph_views = _create_view_graphs(local_bt_graph, config)
    remote_graph_views = _create_view_graphs(remote_bt_graph, config)
    packages_to_skip_dependency_update = set()

    for view_name, local_graph in local_graph_views.items():
        diff_graph: list[ViewPackage] = []
        remote_graph = remote_graph_views[view_name]
        # Created packages
        for path, package in local_graph.items():
            if path not in remote_graph:
                package.state = EntityState.CREATED
                for package_dependency in package.view_dependency_list:
                    package_dependency.state = EntityState.CREATED
                diff_graph.append(package)

        # Deleted packages
        for remote_path, remote_package in remote_graph.items():
            if remote_path not in local_graph:
                remote_package.state = EntityState.DELETED
                for remote_package_dependencies in remote_package.view_dependency_list:
                    remote_package_dependencies.state = EntityState.DELETED
                diff_graph.append(remote_package)
                local_graph[remote_path] = remote_package
                packages_to_skip_dependency_update.add(remote_path)

        # Change dependency state
        for path, package in local_graph.items():
            if path not in remote_graph or path in packages_to_skip_dependency_update:
                continue  # We have already dealt with this case above
            local_dependency_map = package.get_dependency_map()
            remote_dependency_map = remote_graph[path].get_dependency_map()
            for remote_key, remote_value in remote_dependency_map.items():
                # Check if the same key exists in the local_dependency_map
                if remote_key not in local_dependency_map:
                    # we have a dependency that no longer exists in local, package exists but the dependency removed
                    color = EntityState.DELETED
                    dependency_count = 0 - remote_value.dependency_count
                    remote_value = remote_dependency_map[remote_key]

                    remote_dependency_map[remote_key].render_diff = {
                        "from_package": remote_value.from_package,
                        "to_package": remote_value.to_package,
                        "color": color,
                        "label": f"0 ({dependency_count})",
                    }
                    continue

                local_value = local_dependency_map[remote_key]

                # Check if dependency counts are different
                if remote_value.dependency_count != local_value.dependency_count:
                    diff = local_value.dependency_count - remote_value.dependency_count
                    sign = "+" if diff > 0 else ""
                    color = EntityState.CREATED if diff > 0 else EntityState.DELETED
                    dependency_count = (
                        f": {local_value.dependency_count} ({sign}{diff})"
                        if diff != 0
                        else f": {local_value.dependency_count}"
                    )

                    local_dependency_map[remote_key].render_diff = {
                        "from_package": local_value.from_package,
                        "to_package": local_value.to_package,
                        "color": color,
                        "label": f"{dependency_count}",
                    }

            # Created dependencies
            for dependency_path, dependency in local_dependency_map.items():
                if dependency_path not in remote_dependency_map:
                    # we treat a new dependency as a diff
                    color = EntityState.CREATED
                    dependency_count = dependency.dependency_count
                    dependency.render_diff = {
                        "from_package": dependency.from_package,
                        "to_package": dependency.to_package,
                        "color": color,
                        "label": f"{dependency_count} (+{dependency_count})",
                    }

            # Deleted dependencies
            for (
                remote_dependency_path,
                remote_dependency,
            ) in remote_dependency_map.items():
                if remote_dependency_path not in local_dependency_map:
                    remote_dependency.state = EntityState.DELETED
                    remote_dependency.from_package = package
                    remote_dependency.to_package = local_graph[
                        remote_dependency_path
                    ]  # Ensures that the package refs will be in the final graph
                    package.view_dependency_list.append(remote_dependency)

            diff_graph.append(package)

        view_config: dict = config["views"][view_name]
        use_package_path_as_label = view_config.get("usePackagePathAsLabel", True)
        if not use_package_path_as_label:
            _handle_duplicate_name(diff_graph)

        save_to_file(diff_graph, view_name, config)


def _handle_duplicate_name(view_graph: list[ViewPackage]):
    for package in view_graph:
        package_name_split = package.path.split("/")
        found_duplicate = False
        for package_2 in view_graph:
            if package == package_2:
                continue
            package_2_name_split = package_2.path.split("/")
            if package_2_name_split[-1] == package_name_split[-1]:
                if len(package_name_split) >= len(package_2_name_split):
                    package.name = PACKAGE_NAME_SPLITTER.join(package_name_split[-2:])
                else:
                    package.name = package_name_split[-1]
                found_duplicate = True

        if not found_duplicate:
            package.name = package_name_split[-1]


def _create_view_graphs(
    graph: BTGraph, config: dict
) -> dict[str, dict[str, ViewPackage]]:
    bt_packages = graph.get_all_bt_modules_map()
    views = {}

    for view_name, view in config["views"].items():
        view_package_map: dict[str, ViewPackage] = {}
        for bt_package in bt_packages.values():
            view_package = ViewPackage(bt_package)
            view_package_map[view_package.path] = view_package

        for view_package in view_package_map.values():
            view_package.setup_dependencies(view_package_map)

        view_package_map = _filter_packages(view_package_map, view)
        views[view_name] = view_package_map
    return views


def _find_packages_with_depth(
    package: ViewPackage, depth: int, view_package_map: dict[str, ViewPackage]
):
    bt_sub_packages = package.bt_package.get_submodules_recursive()
    filtered_sub_packages = [
        get_view_package_path_from_bt_package(sub_package)
        for sub_package in bt_sub_packages
        if (sub_package.depth - package.bt_package.depth) <= depth
    ]
    return [view_package_map[p] for p in filtered_sub_packages]


def _filter_packages(
    packages_map: dict[str, ViewPackage], view: dict
) -> dict[str, ViewPackage]:
    packages = list(packages_map.values())
    filtered_packages_set: set[ViewPackage] = set()
    # packages
    for package_view in view["packages"]:
        for package in packages:
            filter_path = package_view

            if isinstance(package_view, str):
                if package.path.startswith(filter_path.replace(".", "/")):
                    filtered_packages_set.add(package)

            if isinstance(package_view, dict):
                filter_path = package_view["path"].replace(".", "/")
                filter_path = filter_path.replace("*", "")
                view_depth = package_view["depth"]
                if filter_path == "" and package.parent_path == ".":
                    filtered_packages_set.add(package)
                    depth_filter_packages = _find_packages_with_depth(
                        package, view_depth - 1, packages_map
                    )
                    filtered_packages_set.update(depth_filter_packages)
                elif package.path == filter_path:
                    filtered_packages_set.add(package)
                    depth_filter_packages = _find_packages_with_depth(
                        package, view_depth, packages_map
                    )
                    filtered_packages_set.update(depth_filter_packages)

    if len(view["packages"]) == 0:
        filtered_packages_set = set(packages_map.values())

    # ignorePackages
    updated_filtered_packages_set: set = set()
    for ignore_packages in view["ignorePackages"]:
        for package in filtered_packages_set:
            should_filter = False
            if ignore_packages.startswith("*") and ignore_packages.endswith("*"):
                if ignore_packages[1:-1] in package.path:
                    should_filter = True
            else:
                if package.path.startswith(ignore_packages):
                    should_filter = True

            if not should_filter:
                updated_filtered_packages_set.add(package)

    if len(view["ignorePackages"]) == 0:
        updated_filtered_packages_set = filtered_packages_set

    filtered_packages_set = updated_filtered_packages_set

    for package in filtered_packages_set:
        package.filter_excess_packages_dependencies(filtered_packages_set)
    return {package.path: package for package in filtered_packages_set}



from src.core.bt_module import BTModule
from src.utils.path_manager_singleton import PathManagerSingleton


def get_view_package_path_from_bt_package(bt_package: BTModule) -> str:
    path_manager = PathManagerSingleton()
    raw_name = path_manager.get_relative_path_from_project_root(bt_package.path, True)
    return raw_name



from pathlib import Path


class PathManagerSingleton:
    _instance = None

    _config_path = None
    _config_root_folder_path = None

    _git_config_path = None
    _git_root_folder_path = None

    def __new__(cls):
        if cls._instance is None:
            cls._instance = super().__new__(cls)
        return cls._instance

    def setup(self, config: dict, git_config=None):
        self._set_path(config, git_config=False)
        self._set_path(git_config, git_config=True)

    def get_relative_path_from_project_root(self, path: str, append_root_folder=False):
        if self._config_path is None:
            raise Exception("Should call setup method before any other function")
        try:
            current_config_path = (
                self._config_root_folder_path
                if append_root_folder
                else self._config_path
            )
            return Path(path).relative_to(current_config_path).as_posix()
        except Exception:
            current_config_path = (
                self._git_root_folder_path
                if append_root_folder
                else self._git_config_path
            )
            return Path(path).relative_to(current_config_path).as_posix()

    def _set_path(self, config: dict, git_config: bool):
        if config is None:
            return None
        config_path = Path(config["_config_path"])
        config_root_folder_path = config_path.joinpath(config["rootFolder"])

        config_path = config_path.as_posix()
        config_root_folder_path = config_root_folder_path.as_posix()

        if not git_config:
            self._config_path = config_path
            self._config_root_folder_path = config_root_folder_path
        else:
            self._git_config_path = config_path
            self._git_root_folder_path = config_root_folder_path



import astroid
from astroid.manager import AstroidManager

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from src.core.bt_module import BTModule


class BTFile:
    label: str = ""
    edge_to: list["BTFile"] = None
    ast = None
    module: "BTModule" = None
    am: AstroidManager

    def __init__(self, label: str, module, am: AstroidManager, code_path: str = None):
        self.label = label
        self.ast = None
        self.am = am

        if code_path is not None:
            self.ast: astroid.Module = self.am.ast_from_module_name(code_path)

        self.edge_to = []
        self.module = module

    @property
    def file(self):
        if self.ast:
            return self.ast.file
        return ""

    @property
    def uid(self):
        if self.ast:
            return self.ast.file
        else:
            return self.label

    @property
    def module_path(self) -> str:
        if not self.ast:
            return None
        return "/".join(self.file.split("/")[:-1])

    def __rshift__(self, other):
        if isinstance(other, list):
            existing_edges = set(
                [edge.file for edge in self.edge_to if edge.file != ""]
            )
            new_node_list = filter(lambda e: e.file not in existing_edges, other)
            self.edge_to.extend([node for node in new_node_list])
        else:
            edges = set([edge.file for edge in self.edge_to])
            if other.file in edges:
                return

            self.edge_to.append(other)


def get_imported_modules(
    ast: astroid.Module, root_location: str, am: AstroidManager
) -> list:
    imported_modules = []
    for sub_node in ast.body:
        try:
            if isinstance(sub_node, astroid.node_classes.ImportFrom):
                sub_node: astroid.node_classes.ImportFrom = sub_node

                module_node = am.ast_from_module_name(
                    sub_node.modname,
                    context_file=root_location,
                )
                imported_modules.append(module_node)

            elif isinstance(sub_node, astroid.node_classes.Import):
                for name, _ in sub_node.names:
                    try:
                        module_node = am.ast_from_module_name(
                            name,
                            context_file=root_location,
                        )
                        imported_modules.append(module_node)
                    except Exception:
                        continue
            elif hasattr(sub_node, "body"):
                imported_modules.extend(
                    get_imported_modules(sub_node, root_location, am)
                )

        except astroid.AstroidImportError:
            continue

    return imported_modules



import git


def fetch_git_repo(tmp_dir, github_url, branch):
    repo_path = github_url
    repo = git.Repo.clone_from(repo_path, tmp_dir)

    repo.git.checkout(branch)
    repo.remotes.origin.pull()

    # repo.git.checkout(original_branch)



import os

from src.views.view_entities import ViewPackage
import json


def save_json(view_graph, view_name, config):
    json = _render_json_graph(view_graph, view_name, config)
    project_name = config["name"]
    save_location = os.path.join(config["saveLocation"], f"{project_name}-{view_name}")
    _save_json_file(save_location + ".json", json)


def save_json_diff(view_graph, view_name, config):
    json = _render_json_graph(view_graph, view_name, config)
    project_name = config["name"]
    save_location = os.path.join(
        config["saveLocation"], f"{project_name}-diff-{view_name}"
    )
    _save_json_file(save_location + ".json", json)


def _render_json_graph(view_graph: list[ViewPackage], view_name, config):
    json_packages = [json_package.render_package_json() for json_package in view_graph]

    json_dependencies = [
        relation
        for json_package in view_graph
        for relation in json_package.render_dependency_json()
    ]

    project_name = config.get("name", "")
    title = f"{project_name}-{view_name}"

    json_dict = {"title": title, "packages": json_packages, "edges": json_dependencies}

    if os.getenv("MT_DEBUG"):
        print(json.dumps(json_dict))
        print("Program Complete")
    return json_dict


def _save_json_file(save_location, json_dict):
    os.makedirs(os.path.dirname(save_location), exist_ok=True)
    with open(save_location, "w") as f:
        json.dump(json_dict, f, indent=4)



import astroid
import os

from src.core.bt_file import BTFile
from astroid.manager import AstroidManager


class BTModule:
    parent_module: "BTModule" = None
    child_module: list["BTModule"] = None

    name_if_duplicate_exists = None

    file_list: list["BTFile"] = None

    ast: astroid.Module = None
    am: AstroidManager = None

    def __init__(self, file_path: str, am: AstroidManager) -> None:
        self.ast = am.ast_from_file(file_path)
        self.child_module = []
        self.file_list = []
        self.am = am
        print(f"analyzing {self.path}")

    @property
    def depth(self):
        parents = self.get_parent_module_recursive()
        return len(parents)

    @property
    def name(self):
        return self.ast.name.split(".")[-1]

    @property
    def path(self):
        return os.path.dirname(self.ast.file)

    def add_files(self):
        files = [
            element for element in os.listdir(self.path) if element.endswith(".py")
        ]

        for file in files:
            if file == "testing.py":
                continue
            bt_file = BTFile(label=file.split("/")[-1], module=self, am=self.am)
            bt_file.ast = self.am.ast_from_file(os.path.join(self.path, file))
            self.file_list.append(bt_file)

    def get_files_recursive(self) -> list[BTFile]:
        temp_file_list = self.file_list.copy()
        for child_module in self.child_module:
            temp_file_list.extend(child_module.get_files_recursive())
        return temp_file_list

    def get_submodules_recursive(self) -> list["BTModule"]:
        submodule_list = self.child_module.copy()
        for submodule in self.child_module:
            submodule_list.extend(submodule.get_submodules_recursive())
        return submodule_list

    def get_parent_module_recursive(self):
        if self.parent_module is None:
            return []
        parent_module_list = [self.parent_module]
        parent_module_list.extend(self.parent_module.get_parent_module_recursive())
        return parent_module_list

    def get_module_dependencies(self) -> set["BTModule"]:
        dependencies = set()
        for child in self.file_list:
            dependencies.update(map(lambda e: e.module, child.edge_to))
        return dependencies

    def get_dependency_count(self, other: "BTModule"):
        file_dependencies = other.file_list
        files = [edge for element in self.file_list for edge in element.edge_to]
        count = len([element for element in files if element in file_dependencies])
        return count

    def get_file_level_relations(self, target_module: "BTModule"):
        """
        files contained in this object that depend on :param other:
        :return:
        """

        # relation is a tuple (file, file)
        all_file_relations = [
            (origin_file, target_file)
            for origin_file in self.file_list
            for target_file in origin_file.edge_to
        ]
        relations = [
            relation
            for relation in all_file_relations
            if relation[1] in target_module.file_list
        ]
        return relations



import typer
import json
import os
import requests
import jsonschema
import tempfile
import shutil
from pathlib import Path
from src.providers.json.json_render import save_json, save_json_diff
from src.providers.plantuml.pu_render import save_plant_uml, save_plant_uml_diff
from src.utils.path_manager_singleton import PathManagerSingleton

# from src.utils.functions import verify_config_options
from src.utils.config_manager_singleton import ConfigManagerSingleton

from src.views.view_manager import render_views, render_diff_views

from src.core.bt_graph import BTGraph

from src.git_integration.fetch_git import fetch_git_repo

from astroid.manager import AstroidManager

import astroid

astroid.MANAGER = None

app = typer.Typer(add_completion=True)


@app.command()
def render(config_path: str = "archlens.json"):
    config = read_config_file(config_path)

    mt_path_manager = PathManagerSingleton()
    mt_path_manager.setup(config)

    am = _create_astroid()
    g = BTGraph(am)
    g.build_graph(config)

    render_views(g, config, save_plant_uml)


@app.command()
def render_json(config_path: str = "archlens.json"):
    config = read_config_file(config_path)

    mt_path_manager = PathManagerSingleton()
    mt_path_manager.setup(config)

    am = _create_astroid()
    g = BTGraph(am)
    g.build_graph(config)

    render_views(g, config, save_json)


def _create_astroid():
    am = AstroidManager()
    am.brain["astroid_cache"] = {}
    return am


@app.command()
def render_diff(config_path: str = "archlens.json"):
    with tempfile.TemporaryDirectory() as tmp_dir:
        print("Created temporary directory:", tmp_dir)
        config = read_config_file(config_path)

        fetch_git_repo(tmp_dir, config["github"]["url"], config["github"]["branch"])

        shutil.copyfile(config_path, os.path.join(tmp_dir, "archlens.json"))

        config_git = read_config_file(os.path.join(tmp_dir, "archlens.json"))

        path_manager = PathManagerSingleton()
        path_manager.setup(config, config_git)

        local_am = _create_astroid()
        local_graph = BTGraph(local_am)
        local_graph.build_graph(config)
        # verify_config_options(config, g)

        remote_am = _create_astroid()
        remote_graph = BTGraph(remote_am)
        remote_graph.build_graph(config_git)
        # verify_config_options(config_git, g_git)

        render_diff_views(local_graph, remote_graph, config, save_plant_uml_diff)


@app.command()
def render_diff_json(config_path: str = "archlens.json"):
    with tempfile.TemporaryDirectory() as tmp_dir:
        print("Created temporary directory:", tmp_dir)
        config = read_config_file(config_path)

        fetch_git_repo(tmp_dir, config["github"]["url"], config["github"]["branch"])

        shutil.copyfile(config_path, os.path.join(tmp_dir, "archlens.json"))

        config_git = read_config_file(os.path.join(tmp_dir, "archlens.json"))

        path_manager = PathManagerSingleton()
        path_manager.setup(config, config_git)

        local_am = _create_astroid()
        local_graph = BTGraph(local_am)
        local_graph.build_graph(config)
        # verify_config_options(config, g)

        remote_am = _create_astroid()
        remote_graph = BTGraph(remote_am)
        remote_graph.build_graph(config_git)
        # verify_config_options(config_git, g_git)

        render_diff_views(local_graph, remote_graph, config, save_json_diff)


@app.command()
def init(config_path="./archlens.json"):
    os.makedirs(os.path.dirname(config_path), exist_ok=True)
    template_path = os.path.join(os.path.dirname(__file__), "config.template.json")
    schema = None
    with open(template_path, "r") as f:
        schema = json.load(f)

    schema["name"] = os.path.basename(os.getcwd())
    with open(config_path, "w") as outfile:
        json.dump(schema, outfile, indent=4)


@app.command()
def create_action():
    action_url = "https://raw.githubusercontent.com/archlens/ArchLens/master/.github/workflows/render-diff-on-pr.yml"
    action_path = Path(".github/workflows/render-diff-on-pr.yml")
    typer.secho(f"Creating the action at {action_path}", fg="green")
    action_path.parent.mkdir(parents=True, exist_ok=True)
    action = requests.get(action_url).text

    with open(action_path, "w") as f:
        f.write(action)


def read_config_file(config_path):
    config = None
    with open(config_path, "r") as f:
        config = json.load(f)

    config_schema = None
    schema_path = os.path.join(os.path.dirname(__file__), "config.schema.json")
    with open(schema_path) as fp:
        config_schema = json.load(fp)

    if not os.getenv("MT_DEBUG"):
        jsonschema.validate(instance=config, schema=config_schema)

    config["_config_path"] = os.path.dirname(os.path.abspath(config_path))

    config["saveLocation"] = os.path.normpath(
        os.path.join(config["_config_path"], config["saveLocation"])
    )

    config_manager = ConfigManagerSingleton()
    config_manager.setup(config)

    return config


def main():
    app()


if __name__ == "__main__":
    main()



import os

from src.views.view_entities import ViewPackage
import sys


def save_plant_uml(view_graph, view_name, config):
    plant_uml_str = _render_pu_graph(view_graph, view_name, config)
    project_name = config["name"]
    save_location = os.path.join(config["saveLocation"], f"{project_name}-{view_name}")
    _save_plantuml_str(save_location, plant_uml_str)


def save_plant_uml_diff(diff_graph, view_name, config):
    plant_uml_str = _render_pu_graph(diff_graph, view_name, config)
    project_name = config["name"]
    save_location = os.path.join(
        config["saveLocation"], f"{project_name}-diff-{view_name}"
    )
    _save_plantuml_str(save_location, plant_uml_str)


def _render_pu_graph(view_graph: list[ViewPackage], view_name, config):

    pu_package_string = "\n".join(
        [pu_package.render_package_pu() for pu_package in view_graph]
    )
    pu_dependency_string = "\n".join(
        [pu_package.render_dependency_pu() for pu_package in view_graph]
    )
    project_name = config.get("name", "")
    title = f"{project_name}-{view_name}"
    uml_str = f"""
@startuml
skinparam backgroundColor GhostWhite
title {title}
{pu_package_string}
{pu_dependency_string}
@enduml
        """

    if os.getenv("MT_DEBUG"):
        print(uml_str)
        print("Program Complete")
    return uml_str


def _save_plantuml_str(file_name: str, data: str):
    os.makedirs(os.path.dirname(file_name), exist_ok=True)
    with open(file_name, "w") as f:
        f.write(data)
    python_executable = sys.executable
    plantuml_server = os.getenv(
        "PLANTUML_SERVER_URL",
        "https://www.plantuml.com/plantuml/img/",
    )
    os.system(
        f"{python_executable} -m plantuml --server {plantuml_server}  {file_name}"
    )

    if os.path.exists(file_name):
        os.remove(file_name)



class ConfigManagerSingleton:
    _instance = None

    show_dependency_count = None
    package_color = None

    def __new__(cls):
        if cls._instance is None:
            cls._instance = super().__new__(cls)
        return cls._instance

    def setup(self, config: dict):
        self.show_dependency_count = config.get("showDependencyCount", True)
        self.package_color = config.get("packageColor", "#Azure")



import os
from src.core.bt_graph import BTGraph


def verify_config_options(config: dict, graph: BTGraph):
    views = config.get("views", {})
    root_path = os.path.join(config["_config_path"], config["rootFolder"])
    modules = graph.get_all_bt_modules_map()
    for view_data in views.values():
        packages = view_data.get("packages")
        ignore_packages = [
            element for element in view_data.get("ignorePackages") if "*" not in element
        ]

        total_packages = packages + ignore_packages
        total_packages = total_packages.copy()

        for package in total_packages:
            if isinstance(package, str):
                if os.name == "nt":
                    package = package.replace("/", "\\")
                t = os.path.join(root_path, package)
                if package not in root_path:
                    if t not in modules:
                        raise Exception(
                            f"{package} package from config file does not exist in project"
                        )
            else:
                pack = package["path"]
                if os.name == "nt":
                    pack = package["path"].replace("/", "\\")
                t = os.path.join(root_path, pack)
                if pack not in root_path:
                    name = pack
                    if t not in modules:
                        raise Exception(
                            f"{name} package from config file does not exist in project"
                        )



from setuptools import setup, find_packages, glob


setup(
    name="ArchLens",
    version="0.2",
    description="Designed for visualizing package dependencies and highlighting differences between"
    " branches in GitHub pull requests. It offers customization options to tailor package views.",
    author="The ArchLens Team",
    author_email="mlun@itu.dk",
    url="https://github.com/archlens/ArchLens",
    packages=find_packages(),
    long_description="This is the long description",
    data_files=glob.glob("src/config.**.json"),
    include_package_data=True,
    install_requires=[
        "plantuml",
        "typer",
        "astroid",
        "six",
        "requests",
        "jsonschema",
        "gitpython",
    ],
    classifiers=[
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.9",
    ],
    entry_points={
        "console_scripts": [
            "archlens=src.cli_interface:main",
        ],
    },
)
